// Fl_Icon.cxx
// Copyright (C) 2000 Agenda Computing Inc.
// Written by Derek K.T. Au <phantom@netcom.ca>
// (C) 2001-2004 Brian Webb, Andreas Holzer, Enrique Vidal
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <FL/Fl.H>
#include <FL/fl_draw.H>
#include "Fl_Icon.h"
#include "Fl_XPM.h"
#include "no_pixmap.xpm"

Fl_Icon::Fl_Icon(int x, int y, int w, int h, char* label = 0, 
                 Fl_Pixmap* image = 0) : Fl_Button(x, y, w, h, label) {
	this->image = image ? image : Fl_XPM::copy(no_pixmap_xpm);
	invert = true;
	autoadjust = true;
}

void Fl_Icon::draw() {
	int content_height;

	if (type() == FL_HIDDEN_BUTTON || box() == FL_NO_BOX) return;
	Fl_Color col = value() ? selection_color() : color();
        fl_color( col);
        fl_rectf( x(), y(), w(), h());

	fl_font(labelfont(), labelsize());	

	int data_w, data_h;
	sscanf(image->data[0], "%d %d", &data_w, &data_h);

	content_height = data_h + SEPERATOR + fl_height();	

	int image_x = w() / 2 - data_w / 2;
	int image_y = h() / 2 - content_height / 2;
	
	fl_push_clip(x(), y(), w(), h());
	if(value()) {
		if(invert == true) {
			Fl_Icon::invert_image();
			fl_draw_pixmap(image->data, x() + image_x, 
                                       y() + image_y, selection_color());
			Fl_Icon::invert_image();
		}
		else {
			fl_draw_pixmap(image->data, x() + image_x, 
                                       y() + image_y, selection_color());
		}
	}
	else {
		fl_draw_pixmap(image->data, x()+image_x, y()+image_y, color());
	}
	fl_pop_clip();

	if(value() && invert == true) {
		fl_color(FL_WHITE);
	}
	else {
		fl_color(FL_BLACK);
	}
	if( autoadjust) {
	    char *label_ = new char[strlen(label()) +1];
	    char *newlabel = new char[strlen(label()) +1];
	    strcpy( label_, label());
	    strcpy( newlabel, label_);
	    
	    while( fl_width(label_) > w() && strlen(label_) > 4) {
		strcpy( newlabel, label_);
		strcpy( newlabel+strlen(label_)-4, "...");
		strcpy( label_, newlabel);
	    }
	    
	    label( strdup(newlabel));
	    delete newlabel;
	    autoadjust = false;
	}
	fl_draw(label(), x(), (y() + image_y + data_h + 1), w(), 20, 
                              (Fl_Align)(FL_ALIGN_CENTER | FL_ALIGN_WRAP));
}

void Fl_Icon::invert_pixmap_selection() {
	invert = true;
}

void Fl_Icon::normal_pixmap_selection() {
	invert = false;
}

void  Fl_Icon::invert_image() {
	int w, h, p, q;
	char s, t, id;
	char vs[10];
	unsigned int v;
	unsigned int zq;
	
	sscanf((image->data)[0], "%d %d %d %d", &w, &h, &p, &q);
	for(int i = 1; i < p + 1; i++) {
		sscanf((image->data)[i], "%c\t%c %c", &s, &t, &id);
		if(t == 'c' && id == '#') {
			int l = strlen((image->data)[i]);
			sscanf((image->data)[i], "%c\t%c %c%s", &s,&t,&id, vs);
			v = (unsigned int)strtoul(vs, (char**)(vs + 6), 16);
			zq = ~v;
			zq &= 0xFFFFFF;
			snprintf((char*)((image->data)[i]), l + 1, 
                                         "%c\t%c %c%06x", s, t, id, zq);
		}
	}
}
