// Folder.cxx
// 
// Copyright (C) 2000 Agenda Computing Inc.
// Written by Derek K.T. Au <phantom@netcom.ca>
// (C) 2001-2004 Brian Webb, Andreas Holzer, Enrique Vidal
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.


#include <sys/types.h>
#include <sys/stat.h>
#include <sys/mman.h>
#include <unistd.h>
#include <string.h>
#include <stdlib.h>
#include <dirent.h>
#include <fcntl.h>
#include <stdio.h>
#include <vector>
#include <string>
#include <algorithm>
#include <iostream.h>
#include "Folder.h"
#include "Fl_XPM.h"

class ListFileCallbackData {
public:
  Folder_List_Callback func;
  void* data;

  ListFileCallbackData(Folder_List_Callback func, void* data) {
    this->func = func;
    this->data = data;
  }
};

Folder::Folder(const char* path) {
  this->path = strdup(path);
  icon_list = 0;	
  icon_fd = -1;

  refresh_icon();
}

Folder::~Folder () {
  free(path);
  if(icon_list) {
    munmap(icon_list, icon_length);
  }
  if(icon_fd < 0) {
    close(icon_fd);
  }
}

void Folder::refresh_icon() {
  char* icon_path = (char*)malloc(strlen(path) + 8);
  struct stat sb;

  snprintf(icon_path, strlen(path) + 8, "%s/.icons", path);

  if(icon_fd >= 0) {
    close(icon_fd);
    icon_fd = -1;
  }

  if(icon_fd < 0) {
#ifdef DEMO
    icon_fd = open("/flash/flpad/.icons", O_RDONLY);
#else
    icon_fd = open(icon_path, O_RDONLY);
#endif
  }
  if(icon_fd < 0) {
    return;
  }
  if(icon_list) {
    munmap(icon_list, icon_length);
  }

  fstat(icon_fd, &sb);
  icon_length = sb.st_size;
  icon_list = (char*)mmap(0, sb.st_size, PROT_READ, MAP_SHARED, icon_fd, 0);
}

/* Please use Fl_XPM::delete_pixmap to free the pixmap after u finish using it */
Fl_Pixmap* Folder::get_pixmap(const char* cat, const char* name) {
  char* current;
  char* next;
  char* pixmap_path;
  char* pixmap_pathname;
  Fl_Pixmap* pixmap = 0;
  char* filename = (char*)malloc(strlen(cat) + strlen(name) + 3);

  if(strcmp(cat, ".") == 0) {
    snprintf(filename, strlen(cat) + strlen(name) + 2, "%s;", name);
  }
  else if(strlen(cat) == 0) {      // fixing some root icon names not found
    snprintf(filename, strlen(name) + 2, "%s;", name);
  }
  else {
    snprintf(filename, strlen(cat) + strlen(name) + 3, "%s/%s;", cat, name);
  }

  if(!icon_list) {
    return 0;
  }

  current = icon_list;
  while(current >= icon_list && current < icon_list + icon_length && current) {
    next = index(current, '\n');
    if(strncmp(current, filename, strlen(filename)) == 0 || (!pixmap && strncmp(current, "*;", 2) == 0)) {
      if(strncmp(current, "*;", 2) == 0) {
	pixmap_path = current + 2;
      }
      else {
	pixmap_path = current + strlen(filename);
      }
      pixmap_pathname = (char*)malloc(next - pixmap_path + 1);
      strncpy(pixmap_pathname, pixmap_path, next - pixmap_path);
      pixmap_pathname[next -  pixmap_path] = 0;
      if(pixmap) {
	Fl_XPM::delete_pixmap(pixmap);
      }
      pixmap = Fl_XPM::read(pixmap_pathname);
      free(pixmap_pathname);
    }
    current = next;
    if(current) {
      current++;
    }
  }

  free(filename);

  if(!pixmap) {
    char tmpnam[256], *t;
    unsigned int i;
    for(i = 0, t=tmpnam; (name[i] != '\0') && (i < 251); i++)
      if(isalnum(name[i]))
        *t++ = tolower(name[i]);
      else {
        if((name[i]) == '-') *t++ = name[i];
      }

    *t = '\0';
    char fname[400];
    sprintf(fname, "/usr/share/pixmaps/%s.xpm", tmpnam);
//  fprintf(stderr, "%s\n",fname);
    pixmap = Fl_XPM::read(fname);
  }

  return pixmap;
}
