// LaunchBrowser.h
// This file is header file for LaunchBrowser.cxx
//
// Copyright (C) 2000 Agenda Computing Inc.
// Written by Derek K.T. Au <phantom@netcom.ca>
// (C) 2001-2004 Brian Webb, Andreas Holzer, Enrique Vidal
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

#include <vector>
#include <string>
#include <stack>
#include <FL/Fl.H>
#include <flpda/Fl_Paged_Scroll.H>
#include <FL/Fl_Pixmap.H>
#include "Fl_Icon.h"
#include "Folder.h"

class LB_Action;

class LaunchBrowser : public Fl_Paged_Scroll {
private:
  std::vector<Fl_Pixmap*> pixmap_list;
  std::vector<Fl_Icon*> icon_list;
  std::vector<LB_Action*> callback_list;
  Folder* folder;
  std::string current_folder;
  bool refreshing;
  int cx, cy;

  static void action_cb(Fl_Widget* widget, void* data);

protected:
  int handle(int event);

public:
  std::stack<LB_Action*> action_stack;
  std::string base_path;

  bool sound;

  LaunchBrowser(int x, int y, int w, int h);
  ~LaunchBrowser();

  void home();
  void back();

  void refresh();
  void icons();
  void delete_buttons();
  void add_button(LB_Action *a);

  void category(const std::string &name);

  void do_sound() const;
};

class LB_Action {
public:
  LB_Action(LaunchBrowser *lb, const std::string &label = std::string(""))
    : _lb(lb), _label(label) { }
  virtual ~LB_Action() { }

  virtual LB_Action *copy() const = 0;

  virtual void action() = 0;

  virtual Fl_Pixmap *large_pixmap() const;
  virtual Fl_Pixmap *small_pixmap() const;
  const std::string &label() const {
    return _label;
  }
  LaunchBrowser *launchbrowser() const { return _lb; }

  static Folder *folder;

protected:
  LaunchBrowser *_lb;
  std::string _label;
  std::string _lgpix_fname;
  std::string _smpix_fname;
};
