// XActiveWindows.h
//
// Copyright (C) 2001 Brian Webb (webbb@desertscenes.net)
// (C) 2001-2004 Brian Webb, Andreas Holzer, Enrique Vidal
//
// This program is free software; you can redistribute it and/or
// modify it under the terms of the GNU General Public License
// as published by the Free Software Foundation; either version 2
// of the License, or (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.

#ifndef X_ACTIVE_WINDDOW_H
#define X_ACTIVE_WINDDOW_H

#include <X11/Xlib.h>
#include <vector>
#include <string>

class XActiveWindow {
public:
  XActiveWindow(const std::string &_name = "", const std::string &_command = "",
		Window _xwindowid = 0) :
    name(_name), command(_command), xwindowid(_xwindowid) { }
  XActiveWindow(const XActiveWindow &aw) :
    name(aw.name), command(aw.command), xwindowid(aw.xwindowid) { }
  std::string name;
  std::string command;
  Window xwindowid;
};

class XActiveWindowGen {
public:
  XActiveWindowGen(Display *_display, int screen);
  XActiveWindowGen(const XActiveWindowGen &aw);
  ~XActiveWindowGen() { delete [] root_children; }

  const XActiveWindow &operator*() const { return cur; }
  const XActiveWindowGen &operator++() { next(); return *this; }
  XActiveWindowGen operator++(int) {
    XActiveWindowGen tmp = *this;
    ++(*this);
    return tmp;
  }

  bool done() const { return (cur_window >= nwindows); }

private:
  void next();

  Window *root_children;
  unsigned int nwindows;
  unsigned int cur_window;
  Display *display;
  XActiveWindow cur;
};

#endif
